/*
This file is part of the Juju GUI, which lets users view and manage Juju
environments within a graphical interface (https://launchpad.net/juju-gui).
Copyright (C) 2016 Canonical Ltd.

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU Affero General Public License version 3, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
General Public License for more details.

You should have received a copy of the GNU Affero General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

'use strict';

YUI.add('budget-table-row', function () {

  juju.components.BudgetTableRow = React.createClass({
    propTypes: {
      acl: React.PropTypes.object.isRequired,
      allocationEditable: React.PropTypes.bool,
      extraInfo: React.PropTypes.object,
      listPlansForCharm: React.PropTypes.func,
      plansEditable: React.PropTypes.bool,
      service: React.PropTypes.object.isRequired,
      showExtra: React.PropTypes.bool,
      withPlans: React.PropTypes.bool
    },

    plansXHR: null,

    getInitialState: function getInitialState() {
      return {
        editAllocation: false,
        expanded: false,
        plansLoading: false,
        plans: []
      };
    },

    componentWillMount: function componentWillMount() {
      this._getPlans();
    },

    componentWillUnmount: function componentWillUnmount() {
      if (this.plansXHR) {
        this.plansXHR.abort();
      }
    },

    /**
      Get the list of plans available for the service.
      @method _getPlans
    */
    _getPlans: function _getPlans() {
      var _this = this;

      this.setState({ plansLoading: true }, function () {
        _this.plansXHR = _this.props.listPlansForCharm(_this.props.service.get('charm'), _this._getPlansCallback);
      });
    },

    /**
      Callback for when plans for an entity have been successfully fetched.
       @method _getPlansCallback
      @param {String} error An error message, or null if there's no error.
      @param {Array} plans A list of the plans found.
    */
    _getPlansCallback: function _getPlansCallback(error, plans) {
      if (error) {
        console.error('Fetching plans failed: ' + error);
      } else {
        this.setState({
          plansLoading: false,
          plans: plans
        });
      }
    },

    /**
     Toggle the expanded state.
      @method _toggle
    */
    _toggle: function _toggle() {
      this.setState({ expanded: !this.state.expanded });
    },

    /**
     Toggle the allocation field state.
      @method _toggleAllocation
    */
    _toggleAllocation: function _toggleAllocation() {
      this.setState({ editAllocation: !this.state.editAllocation });
    },

    /**
     Generate the change plan form.
      @method _generatePlans
     @returns {Object} The plan form.
    */
    _generatePlans: function _generatePlans() {
      var _this2 = this;

      var disabled = this.props.acl.isReadOnly();
      var plans = this.state.plans.map(function (plan, i) {
        return React.createElement(
          'li',
          { className: 'budget-table__plan twelve-col',
            key: i },
          React.createElement(
            'div',
            { className: 'six-col' },
            React.createElement(
              'h4',
              null,
              plan.url
            ),
            React.createElement(
              'p',
              null,
              plan.description
            )
          ),
          React.createElement(
            'div',
            { className: 'two-col' },
            plan.price
          ),
          React.createElement(
            'div',
            { className: 'two-col' },
            'Recommended allocation: $550.'
          ),
          React.createElement(
            'div',
            { className: 'two-col last-col' },
            React.createElement(juju.components.GenericButton, {
              action: _this2._toggle,
              disabled: disabled,
              type: 'neutral',
              title: 'Select plan' })
          )
        );
      });
      return React.createElement(
        'ul',
        { className: 'budget-table__plans twelve-col no-margin-bottom' },
        plans
      );
    },

    /**
     Generate the change plan form.
      @method _generateChangePlan
     @returns {Object} The plan form.
    */
    _generateChangePlan: function _generateChangePlan() {
      if (!this.props.plansEditable || this.state.plans && this.state.plans.length === 0) {
        return;
      }
      return React.createElement(
        'div',
        null,
        React.createElement(
          'div',
          { className: 'budget-table__current twelve-col no-margin-bottom' },
          this._generateSharedFields()
        ),
        this._generatePlans(),
        React.createElement(
          'p',
          { className: 'budget-table__plan-notice twelve-col' },
          'By setting an allocation and selecting a plan you agree to the plans terms and conditions'
        )
      );
    },

    /**
     Generate the edit button if editable.
      @method _generateEdit
     @returns {Object} The edit component.
    */
    _generateEdit: function _generateEdit() {
      if (!this.props.plansEditable || this.state.plans.length === 0) {
        return;
      }
      var disabled = this.props.acl.isReadOnly();
      return React.createElement(
        'div',
        { className: 'two-col last-col no-margin-bottom' },
        React.createElement(
          'div',
          { className: 'budget-table__edit' },
          React.createElement(juju.components.GenericButton, {
            action: this._toggle,
            disabled: disabled,
            type: 'neutral',
            title: 'Change plan' })
        )
      );
    },

    /**
     Generate the input or display for the allocation.
      @method _generateAllocation
     @returns {Object} The allocation markup.
    */
    _generateAllocation: function _generateAllocation() {
      if (this.props.allocationEditable && this.state.editAllocation) {
        return React.createElement('input', { className: 'budget-table-row__allocation-input',
          type: 'text',
          value: '$1' });
      } else {
        return React.createElement(
          'span',
          { onClick: this.props.allocationEditable ? this._toggleAllocation : undefined },
          '$1'
        );
      }
    },

    /**
     Generate the shared fields.
      @method _generateSharedFields
     @returns {Object} The fields markup.
    */
    _generateSharedFields: function _generateSharedFields() {
      var service = this.props.service;
      return React.createElement(
        'div',
        null,
        React.createElement(
          'div',
          { className: 'three-col no-margin-bottom' },
          React.createElement('img', { className: 'budget-table__charm-icon',
            src: service.get('icon') }),
          service.get('name')
        ),
        React.createElement(
          'div',
          { className: 'one-col no-margin-bottom' },
          service.get('unit_count')
        )
      );
    },

    /**
      Generate the details for the selected plan.
       @method _generateSelectedPlan
      @returns {Object} The plan markup.
    */
    _generateSelectedPlan: function _generateSelectedPlan() {
      var service = this.props.service;
      var activePlan = service.get('activePlan');
      if (activePlan) {
        return React.createElement(
          'span',
          null,
          activePlan.url,
          ' (',
          activePlan.price,
          ')'
        );
      } else if (this.state.plans.length > 0) {
        return React.createElement(
          'span',
          null,
          'You need to select a plan'
        );
      } else {
        return React.createElement(
          'span',
          null,
          '-'
        );
      }
    },

    /**
      Generate the extra info section.
       @method _generateExtra
      @returns {Object} The extra info markup.
    */
    _generateExtra: function _generateExtra() {
      if (!this.props.showExtra) {
        return;
      }
      return React.createElement(
        'div',
        { className: 'twelve-col no-margin-bottom' },
        this.props.extraInfo
      );
    },

    /**
      Generate plan cols.
       @method _generatePlanCols
      @returns {Object} The plan cols markup.
    */
    _generatePlanCols: function _generatePlanCols() {
      if (!this.props.withPlans) {
        return;
      }
      var plansEditable = this.props.plansEditable;
      var editableWidth = !plansEditable ? 'two-col' : 'one-col';
      return React.createElement(
        'div',
        null,
        React.createElement(
          'div',
          { className: 'three-col no-margin-bottom' },
          this._generateSelectedPlan()
        ),
        React.createElement(
          'div',
          { className: editableWidth + ' no-margin-bottom' },
          '$1'
        ),
        React.createElement(
          'div',
          { className: editableWidth + ' no-margin-bottom' },
          this._generateAllocation()
        ),
        React.createElement(
          'div',
          { className: 'one-col no-margin-bottom' + (plansEditable ? '' : ' last-col') },
          '$1'
        ),
        this._generateEdit()
      );
    },

    render: function render() {
      var classes = {
        'budget-table-row': true,
        'twelve-col': true
      };
      return React.createElement(
        juju.components.ExpandingRow,
        {
          classes: classes,
          clickable: false,
          expanded: this.state.expanded },
        React.createElement(
          'div',
          null,
          this._generateSharedFields(),
          this._generatePlanCols(),
          this._generateExtra()
        ),
        React.createElement(
          'div',
          null,
          this._generateChangePlan()
        )
      );
    }

  });
}, '0.1.0', {
  requires: ['expanding-row', 'generic-button']
});